# coding: utf-8
# Project Configuration Information

# project info
__version__ = "0.0.20"
__index_page__ = "https://github.com/CzaOrz/loggus"
__doc_page__ = "https://github.com/CzaOrz/loggus/README.md"
# author info
__author__ = "https://github.com/CzaOrz"
__email__ = "972542655@qq.com"
__phone__ = "*** **** ****"
